package com.flyfish.framework.adaptor;

import com.flyfish.framework.adaptor.spi.CriteriaAdaptorFactory;
import org.springframework.core.io.support.SpringFactoriesLoader;
import org.springframework.dao.InvalidDataAccessApiUsageException;

import java.util.List;

/**
 * 可以通过该工具类快速创建适配器
 *
 * @author wangyu
 */
public final class CriteriaAdaptors {

    private static final List<CriteriaAdaptorFactory> FACTORIES =
            SpringFactoriesLoader.loadFactories(CriteriaAdaptorFactory.class, null);

    public static CriteriaAdaptor getAdaptor() {
        return FACTORIES.stream()
                .findFirst()
                .map(CriteriaAdaptorFactory::produce)
                .orElseThrow(() -> new InvalidDataAccessApiUsageException("未实现当前查询的适配工厂！"));
    }


    public static CriteriaAdaptor getAdaptor(Object criteria) {
        return FACTORIES.stream()
                .filter(factory -> factory.criteriaType().isAssignableFrom(criteria.getClass()))
                .findFirst()
                .map(CriteriaAdaptorFactory::produce)
                .orElseThrow(() -> new InvalidDataAccessApiUsageException("未实现当前查询的适配工厂！"));
    }
}
