package com.flyfish.framework.domain.base;

import com.flyfish.framework.annotations.query.QueryField;
import com.flyfish.framework.builder.CriteriaBuilder;
import com.flyfish.framework.query.Queries;
import lombok.Getter;
import lombok.Setter;
import org.springframework.data.domain.Sort;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.util.StringUtils;

import java.util.Collection;
import java.util.List;

/**
 * 基于名字模糊匹配的查询实体
 */
@Getter
@Setter
public class NameLikeQo<T extends Domain> extends BaseQo<T> {

    @QueryField(type = QueryField.Type.LIKE)
    protected String name;

    @QueryField
    protected String code;

    @QueryField(name = "createTime", type = QueryField.Type.BETWEEN)
    protected List<String> createTimeRange;

    @QueryField(name = "modifyTime", type = QueryField.Type.BETWEEN)
    protected List<String> modifyTimeRange;

    @QueryField
    protected String creatorId;

    @QueryField
    protected String modifierId;

    @QueryField
    protected Boolean enable;

    @QueryField(type = QueryField.Type.IN)
    protected Collection<String> ids;

    @QueryField(name = "id", type = QueryField.Type.NE)
    protected String excludeId;

    /**
     * 排序参数
     */
    private String sort;

    /**
     * 去重字段
     */
    private String distinct;

    @Override
    public CriteriaBuilder<T> criteriaBuilder() {
        return super.criteriaBuilder()
                .with("name", CriteriaBuilder.Builders.LIKE)
                .with("enable", "code", "creatorId", "modifierId")
                .with("ids", "id", CriteriaBuilder.Builders.IN)
                .with("excludeId", "id", Criteria::ne)
                .with("createTimeRange", "createTime", CriteriaBuilder.Builders.DATE_RANGE)
                .with("modifyTimeRange", "modifyTime", CriteriaBuilder.Builders.DATE_RANGE);
    }


    @Override
    public Sort sorts() {
        if (StringUtils.isEmpty(sort)) {
            return super.sorts();
        }
        return Sort.by(Sort.Order.desc(sort));
    }
}
